class User < ActiveRecord::Base
  devise :database_authenticatable, :rememberable, :trackable

  before_create :generate_token
  after_create :send_activate_email

  validates_presence_of :fname, message: "Поле Имя не может быть пустым"
  validates_presence_of :mname, message: "Поле Отчество не может быть пустым"

  validates_presence_of   :email, message: "Поле Email не может быть пустым"
  validates_uniqueness_of :email, message: "Данный Email уже зарегистрирован"
  validates_format_of     :email, with: /\A[^@]+@([^@\.]+\.)+[^@\.]+\z/, message: "Неверный формат Email"

  validates_presence_of     :password, on: :update, message: "Пароль не может быть пустым", :if => :password_required?
  validates_confirmation_of :password, message: "Пароли не совпадают"
  validates_length_of       :password, within: 0..128, on: :create, allow_blank: true
  validates_length_of       :password, within: 6..128, on: :update, message: "Пароль должен быть не менее 6 символов", :if => :password_required?

  attr_accessor :pass_required

  def send_activate_email
    AppMailer.user_activate(self).deliver
  end

  def generate_token
    self.token = loop do
      random_token = SecureRandom.urlsafe_base64(nil, false).slice!(0...6)
      break random_token unless User.exists?(token: random_token)
    end
  end

  def password_required?
    pass_required == nil ? true : pass_required
  end

end